package broker

import (
	"fmt"

	"github.com/pivotal-cf/brokerapi"
)

func (b *Broker) Provision(instanceID string, details brokerapi.ProvisionDetails, asyncAllowed bool) (brokerapi.ProvisionedServiceSpec, error) {
	if !asyncAllowed {
		return brokerapi.ProvisionedServiceSpec{}, brokerapi.ErrAsyncRequired
	}

	requestParams, err := convertProvisionDetailsToMap(details)
	if err != nil {
		return brokerapi.ProvisionedServiceSpec{}, err
	}

	isAsync, err := b.deployInstance(instanceID, details.PlanID, "", requestParams, false)

	return brokerapi.ProvisionedServiceSpec{IsAsync: isAsync}, convertToDisplayableError(err, b.logger)
}

func (b *Broker) deployInstance(instanceID, planID, previousPlanID string, arbitraryParams map[string]interface{}, replace bool) (bool, error) {
	errs := func(err error) (bool, error) {
		b.logger.Println(err)
		return true, err
	}

	planConfig := b.serviceOffering.FindPlanById(planID)
	if planConfig == nil {
		return errs(DisplayableError{fmt.Errorf("plan %s not found", planID)})
	}

	if !replace {
		exists, err := b.instanceRepo.InstanceExists(instanceID)
		if err != nil {
			return errs(fmt.Errorf("error fetching instance: %s", err))
		}
		if exists {
			return false, DisplayableError{brokerapi.ErrInstanceAlreadyExists}
		}

		count, err := b.instanceRepo.CountByPlanID(planConfig.ID)
		if err != nil {
			return errs(fmt.Errorf("error counting plans: %s", err))
		}

		if planConfig.Quotas != nil && count >= planConfig.Quotas.ServiceInstanceLimit {
			return errs(DisplayableError{brokerapi.ErrPlanQuotaExceeded})
		}
	}

	_, err := b.deployer.Deploy(Instance{ID: instanceID, PlanID: planConfig.ID}, planID, deploymentName(instanceID), Create, arbitraryParams, nil, nil)
	if err != nil {
		return errs(err)
	}

	return true, nil
}
